@@ -1,8 +1,8 @@
-/*  $Id: AfternoonStalkerEngine.cpp,v 1.20 2003/01/04 11:18:16 sarrazip Exp $
+/*  $Id: AfternoonStalkerEngine.cpp,v 1.28 2004/05/02 04:55:45 sarrazip Exp $
     AfternoonStalkerEngine.cpp - A robot-killing video game engine.
 
     afternoonstalker - A robot-killing video game.
-    Copyright (C) 2001, 2002, 2003 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -24,7 +24,7 @@
 
 #include "RobotSprite.h"
 
-#include <gengameng/PixmapLoadError.h>
+#include <flatzebra/PixmapLoadError.h>
 
 #include <assert.h>
 #include <stdlib.h>
@@ -65,6 +65,8 @@ using namespace std;
 #include "images/wall_end_tile_1.xpm"
 #include "images/wall_end_tile_2.xpm"
 #include "images/wall_end_tile_3.xpm"
+#include "images/bomb_explosion_0.xpm"
+#include "images/bomb_explosion_1.xpm"
 
 
 /*  Sprite images:
@@ -105,6 +107,7 @@ using namespace std;
 #include "images/invisible_robot_explosion_1.xpm"
 
 #include "images/gun.xpm"
+#include "images/bomb.xpm"
 #include "images/spider_right_0.xpm"
 #include "images/spider_right_1.xpm"
 #include "images/spider_up_0.xpm"
@@ -147,7 +150,7 @@ using namespace std;
 //
 
 static const int
-    FRAMES_PER_SECOND = 20,
+    FPS = 20,  // frames per second (nominal)
     TILE_SIDE = 32,
     SCORE_TTL = 20,
     NUM_INIT_LIVES = 5,
@@ -157,7 +160,9 @@ static const int
     SPIDER_SPEED_FACTOR = 2,
     BAT_SPEED_FACTOR = 3,
     BULLET_SPEED_FACTOR = 8,
-    BULLETS_PER_GUN = 6;
+    BULLETS_PER_GUN = 6,
+    SPIDER_SCORE = 100,
+    BAT_SCORE = 300;
 
 static const double PI = 4.0 * atan(1.0);
 static const double TWO_PI = 2.0 * PI;
@@ -257,8 +262,12 @@ countDirections(const bool d[4])
 
 AfternoonStalkerEngine::AfternoonStalkerEngine(
 			    const string &windowManagerCaption,
-			    bool useSound, bool pedantic, bool fullScreen)
-						throw(int, string)
+			    bool useSound,
+			    bool pedantic,
+			    bool fullScreen,
+			    bool onlyBlueRobots,
+			    const string &setFilename)
+				throw(int, string)
 
   : GameEngine(Couple(640, 416), windowManagerCaption, fullScreen),
 		// may throw string exception
@@ -284,6 +293,7 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
     lastPlayerDir(-1),
     lastRequestedDir(-1),
     playerParalysisTime(0),
+    playerHasBomb(false),
 
     playerBulletPA(1),
     playerBulletSprites(),
@@ -294,6 +304,15 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
     gunPA(1),
     gunSprites(),
 
+    bombPA(1),
+    bombSprites(),
+    timeUntilNextBomb(0),
+
+    theBomb(),
+    bombExplosionTicks(0),
+    bombExplosionPA(2),
+    bombExplosionSprites(),
+
     grayRobotPA(2),
     blueRobotPA(2),
     whiteRobotPA(2),
@@ -302,6 +321,7 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
     blinkPA(1),
     robotSprites(),
     timeUntilNextRobot(0),
+    sendOnlyBlueRobots(onlyBlueRobots),
     initRobotPos(),
     numCreatedRobots(0),
     maxNumRobots(0),
@@ -332,20 +352,24 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
     digitPA(10),
     scoreSprites(),
 
-    escapePressed(false),
-    leftArrowPressed(false),
-    rightArrowPressed(false),
-    upArrowPressed(false),
-    downArrowPressed(false),
-    spacePressed(false),
-    spacePressedBefore(false),
-    ctrlPressed(false),
-    ctrlPressedBefore(false),
-    letterPPressed(false),
-    letterPPressedBefore(false),
+    quitKS(SDLK_ESCAPE),
+    leftKS(SDLK_LEFT),
+    rightKS(SDLK_RIGHT),
+    upKS(SDLK_UP),
+    downKS(SDLK_DOWN),
+    startKS(SDLK_SPACE),
+    shootKS(SDLK_LCTRL, SDLK_RCTRL),
+    pauseKS(SDLK_p),
+    bombKS(SDLK_b),
+    shootLeftKS(SDLK_h),
+    shootRightKS(SDLK_l),
+    shootUpKS(SDLK_k),
+    shootDownKS(SDLK_j),
 
     theSoundMixer(NULL)
 {
+    this->useSound = useSound;
+
     blackColor = SDL_MapRGB(theSDLScreen->format, 0, 0, 0);
 
     try
@@ -360,7 +384,7 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
 	    throw msg;
 	}
 
-	doOneTimeInitializations(useSound, pedantic);
+	doOneTimeInitializations(useSound, pedantic, setFilename);
     }
     catch (string &msg)
     {
@@ -412,6 +436,7 @@ AfternoonStalkerEngine::initializePlayerLife()
     deleteSprites(playerBulletSprites);
     deleteSprites(playerExplosionSprites);
     deleteSprites(gunSprites);
+    deleteSprites(bombSprites);
     deleteSprites(robotSprites);
     deleteSprites(robotBulletSprites);
     deleteSprites(robotExplosionSprites);
@@ -419,23 +444,38 @@ AfternoonStalkerEngine::initializePlayerLife()
     deleteSprites(spiderExplosionSprites);
     deleteSprites(batSprites);
     deleteSprites(batExplosionSprites);
+    deleteSprites(bombExplosionSprites);
 
+    setTimeUntilNextBomb();
     setTimeUntilNextRobot();
     setTimeUntilNextAnimal();
+
+    theBomb.reset();
+    bombExplosionTicks = 0;
+}
+
+
+void
+AfternoonStalkerEngine::setTimeUntilNextBomb()
+{
+    if (theScore >= 30000)
+	timeUntilNextBomb = Rnd(FPS * 2, FPS * 5);
+    else
+	timeUntilNextBomb = Rnd(FPS * 45, FPS * 120);
 }
 
 
 void
 AfternoonStalkerEngine::setTimeUntilNextRobot()
 {
-    timeUntilNextRobot = FRAMES_PER_SECOND * 2;
+    timeUntilNextRobot = Rnd(FPS * 1, FPS * 3);
 }
 
 
 void
 AfternoonStalkerEngine::setTimeUntilNextAnimal()
 {
-    timeUntilNextAnimal = FRAMES_PER_SECOND * 5;
+    timeUntilNextAnimal = Rnd(FPS * 3, FPS * 7);
 }
 
 
@@ -459,15 +499,17 @@ getDir(const char *defaultValue, const char *envVarName)
 
 void
 AfternoonStalkerEngine::doOneTimeInitializations(
-				bool useSound, bool pedantic) throw(string)
+				bool useSound,
+				bool pedantic,
+				const string &setFilename)
+					throw(string)
 /*  Throws an error message in a 'string' if an error occurs.
 */
 {
     string pkgdatadir = getDir(PKGDATADIR, "PKGDATADIR");
     string pkgsounddir = getDir(PKGSOUNDDIR, "PKGSOUNDDIR");
 
-    string setFilename = pkgdatadir + "level1.set";
-    loadLevel(1, setFilename);
+    loadLevel(1, pkgdatadir + setFilename);
     assert(initPlayerPos.x >= 0);
     assert(initPlayerPos.y >= 0);
 
@@ -482,7 +524,15 @@ AfternoonStalkerEngine::doOneTimeInitializations(
     */
     if (useSound)
     {
-	theSoundMixer = new SoundMixer();  // may throw string
+	try
+	{
+	    theSoundMixer = NULL;
+	    theSoundMixer = new SoundMixer();
+	}
+	catch (const SoundMixer::Error &e)
+	{
+	    return;
+	}
 
 	try
 	{
@@ -495,6 +545,7 @@ AfternoonStalkerEngine::doOneTimeInitializations(
 	    batKilledSound.init(pkgsounddir + "bat-killed.wav");
 	    spiderKilledSound.init(pkgsounddir + "spider-killed.wav");
 	    newLifeSound.init(pkgsounddir + "new-life.wav");
+	    bombExplodesSound.init(pkgsounddir + "bomb-explodes.wav");
 	}
 	catch (const SoundMixer::Error &e)
 	{
@@ -529,7 +580,7 @@ AfternoonStalkerEngine::loadLevel(int levelNo, const string &setFilename)
 {
     ifstream setFile(setFilename.c_str());
     if (!setFile)
-	throw "Could not open " + setFilename;
+	throw string("Could not open " + setFilename);
     
     size_t levelWidth = 0, levelHeight = 0;
     string line;
@@ -539,18 +590,18 @@ AfternoonStalkerEngine::loadLevel(int levelNo, const string &setFilename)
     */
 
     if (!getline(setFile, line))
-	throw "Could not read level width from set file";
+	throw string("Could not read level width from set file");
     if (sscanf(line.c_str(), "%u", &levelWidth) != 1)
-	throw "Level width expected";
+	throw string("Level width expected");
     if (levelWidth < 3 || levelWidth > 255)  // arbitrary safety limit
-	throw "Invalid level width";
+	throw string("Invalid level width");
 
     if (!getline(setFile, line))
-	throw "Could not read level height from set file";
+	throw string("Could not read level height from set file");
     if (sscanf(line.c_str(), "%u", &levelHeight) != 1)
-	throw "Could not read level height";
+	throw string("Could not read level height");
     if (levelHeight < 3 || levelHeight > 255)  // arbitrary safety limit
-	throw "Invalid level height";
+	throw string("Invalid level height");
 
 
     /*  Read tile characters from the file and initialize tileMatrix
@@ -565,9 +616,9 @@ AfternoonStalkerEngine::loadLevel(int levelNo, const string &setFilename)
     for (y = 0; y < levelHeight; y++)
     {
 	if (!getline(setFile, line))
-	    throw "Could not read level row";
+	    throw string("Could not read level row");
 	if (line.length() != levelWidth)
-	    throw "Level row has wrong width";
+	    throw string("Level row has wrong width");
 
 	tileMatrix.push_back(TileMatrixRow(levelWidth));
 
@@ -601,7 +652,7 @@ AfternoonStalkerEngine::loadLevel(int levelNo, const string &setFilename)
 		    			+ Couple(1, 1 + TILE_MATRIX_Y_OFFSET);
 		    break;
 
-		default: throw "Invalid character in level row";
+		default: throw string("Invalid character in level row");
 	    }
 
 	    if (noBunker != NULL && tileNo == BUNKER_TILE)
@@ -617,7 +668,7 @@ AfternoonStalkerEngine::loadLevel(int levelNo, const string &setFilename)
 
 
     if (initPlayerPos == Couple(-1, -1))
-	throw "no bunker door tile in level";
+	throw string("no bunker door tile in level");
 
 
     /*  Find the left-most floor tile of the next-to-last row.
@@ -627,7 +678,7 @@ AfternoonStalkerEngine::loadLevel(int levelNo, const string &setFilename)
     TileMatrixRow::iterator it =
 			find(lastRow.begin(), lastRow.end(), FLOOR_TILE);
     if (it == lastRow.end())
-	throw "no floor tile on next-to-last row";
+	throw string("no floor tile on next-to-last row");
 
     x = distance(lastRow.begin(), it);
     assert(x < levelWidth);
@@ -649,11 +700,11 @@ AfternoonStalkerEngine::loadLevel(int levelNo, const string &setFilename)
     TileMatrixRow &firstRow = tileMatrix[1];
     it = find(firstRow.begin(), firstRow.end(), COBWEB_TILE);
     if (it == firstRow.end())
-	throw "no cobweb tile on second row";
+	throw string("no cobweb tile on second row");
     it++;
     it = find(it, firstRow.end(), COBWEB_TILE);
     if (it == firstRow.end())
-	throw "only one cobweb tile on second row";
+	throw string("only one cobweb tile on second row");
     x = distance(firstRow.begin(), it);
     assert(x < levelWidth);
 
@@ -672,57 +723,31 @@ AfternoonStalkerEngine::loadLevel(int levelNo, const string &setFilename)
 ///////////////////////////////////////////////////////////////////////////////
 
 
-/*virtual*/ void
+/*virtual*/
+void
 AfternoonStalkerEngine::processKey(SDLKey keysym, bool pressed)
 {
-    switch (keysym)
-    {
-	case SDLK_ESCAPE:
-	    escapePressed = pressed;
-	    break;
-	case SDLK_LEFT:
-	    leftArrowPressed = pressed;
-	    if (pressed)
-		rightArrowPressed = false;
-	    break;
-	case SDLK_RIGHT:
-	    rightArrowPressed = pressed;
-	    if (pressed)
-		leftArrowPressed = false;
-	    break;
-	case SDLK_UP:
-	    upArrowPressed = pressed;
-	    if (pressed)
-		downArrowPressed = false;
-	    break;
-	case SDLK_DOWN:
-	    downArrowPressed = pressed;
-	    if (pressed)
-		upArrowPressed = false;
-	    break;
-	case SDLK_SPACE:
-	    spacePressed = pressed;
-	    break;
-	case SDLK_LCTRL:
-	    ctrlPressed = pressed;
-	    break;
-	case SDLK_RCTRL:
-	    ctrlPressed = pressed;
-	    break;
-	case SDLK_p:
-	    letterPPressed = pressed;
-	    break;
-
-	default:
-	    ;  // ignore other keys
-    }
+    quitKS.check(keysym, pressed);
+    leftKS.check(keysym, pressed);
+    rightKS.check(keysym, pressed);
+    upKS.check(keysym, pressed);
+    downKS.check(keysym, pressed);
+    startKS.check(keysym, pressed);
+    shootKS.check(keysym, pressed);
+    pauseKS.check(keysym, pressed);
+    bombKS.check(keysym, pressed);
+    shootLeftKS.check(keysym, pressed);
+    shootDownKS.check(keysym, pressed);
+    shootUpKS.check(keysym, pressed);
+    shootRightKS.check(keysym, pressed);
 }
 
 
-/*virtual*/ bool
+/*virtual*/
+bool
 AfternoonStalkerEngine::tick()
 {
-    if (escapePressed)
+    if (quitKS.isPressed())
 	return false;
 
     if (paused)
@@ -732,13 +757,12 @@ AfternoonStalkerEngine::tick()
 	restoreForeground();
 	drawScoreBoard();
 
-	if (letterPPressed && !letterPPressedBefore)
+	if (pauseKS.justPressed())
 	{
 	    paused = false;
-	    displayPauseMessage(false);
 	}
 	else
-	    displayPauseMessage(true);
+	    displayPauseMessage();
     }
     else
     {
@@ -754,15 +778,14 @@ AfternoonStalkerEngine::tick()
 
 	if (numPlayerLives == 0)
 	{
-	    if (!spacePressed && spacePressedBefore)
+	    if (startKS.justPressed())
 	    {
-		initializeLevel();
 		theScore = getenv("INITSCORE") ? atol(getenv("INITSCORE")) : 0;
+		initializeLevel();
 		addToNumPlayerLives(5);
-		displayStartMessage(false);
 	    }
 	    else
-		displayStartMessage(true);
+		displayStartMessage();
 	}
 	
 
@@ -799,10 +822,15 @@ AfternoonStalkerEngine::tick()
 	}
     }
 
-    spacePressedBefore = spacePressed;
-    ctrlPressedBefore = ctrlPressed;
-    letterPPressedBefore = letterPPressed;
-    
+    startKS.remember();
+    shootKS.remember();
+    pauseKS.remember();
+    bombKS.remember();
+    shootLeftKS.remember();
+    shootRightKS.remember();
+    shootUpKS.remember();
+    shootDownKS.remember();
+
     return true;
 }
 
@@ -812,10 +840,10 @@ AfternoonStalkerEngine::animatePlayer()
 /*  Returns true if the game must continue, or false to have it stop.
 */
 {
-    if (letterPPressed && !letterPPressedBefore)
+    if (pauseKS.justPressed())
     {
 	paused = true;
-	displayPauseMessage(true);
+	displayPauseMessage();
 	return true;
     }
 
@@ -839,30 +867,64 @@ AfternoonStalkerEngine::animatePlayer()
     /*  Register the last requested direction, which will be the direction
 	of the player's bullet, when applicable.
     */
-    if (rightArrowPressed)
+    if (rightKS.isPressed())
 	lastRequestedDir = RIGHT;
-    if (upArrowPressed)
+    else if (upKS.isPressed())
 	lastRequestedDir = UP;
-    if (leftArrowPressed)
+    else if (leftKS.isPressed())
 	lastRequestedDir = LEFT;
-    if (downArrowPressed)
+    else if (downKS.isPressed())
 	lastRequestedDir = DOWN;
-    if (ctrlPressed && !ctrlPressedBefore)
-	makePlayerShoot();
 
+    if (shootKS.justPressed())
+	makePlayerShoot(lastRequestedDir == -1 ? UP : lastRequestedDir);
+    else if (shootLeftKS.justPressed())
+	makePlayerShoot(LEFT);
+    else if (shootRightKS.justPressed())
+	makePlayerShoot(RIGHT);
+    else if (shootUpKS.justPressed())
+	makePlayerShoot(UP);
+    else if (shootDownKS.justPressed())
+	makePlayerShoot(DOWN);
+
+    if (bombKS.justPressed())
+    {
+	if (playerHasBomb)
+	    layPlayerBomb();
+	else
+	    playSoundEffect(shootingBlanksSound);
+    }
 
     playerSprite->currentPixmapIndex = 0;
 	    // use "stopped" image unless a non-zero speed is chosen
 
-    bool attemptedDirs[4] =
-    {
-	rightArrowPressed, upArrowPressed, leftArrowPressed, downArrowPressed
-    };
-    playerSpeed = attemptMove(*playerSprite,
-				attemptedDirs,
-				PLAYER_SPEED_FACTOR);
+    bool desiredDirs[4] =
+	{
+	    rightKS.isPressed(), upKS.isPressed(),
+	    leftKS.isPressed(), downKS.isPressed(),
+	};
+    playerSpeed = attemptMove(*playerSprite, desiredDirs, PLAYER_SPEED_FACTOR);
     if (playerSpeed.isZero())
-        return true;
+    {
+	if (desiredDirs[RIGHT] || desiredDirs[UP]
+		    || desiredDirs[LEFT] || desiredDirs[DOWN])
+	{
+	    /*
+		If the user is pressing at least one arrow, then try to
+		reuse the last non zero direction.
+		This can be is useful to turn corners automatically.
+	    */
+	    bool backupDirs[4] = { false, false, false, false };
+	    backupDirs[lastPlayerDir] = true;
+	    playerSpeed = attemptMove(*playerSprite,
+				    backupDirs, PLAYER_SPEED_FACTOR);
+	    if (playerSpeed.isZero())
+		return true;
+	}
+	else
+	    return true;
+    }
+
 
     playerSprite->currentPixmapIndex = 1 + ((tickCount & 2) != 0);
 
@@ -879,7 +941,11 @@ AfternoonStalkerEngine::animatePlayer()
 
 
 void
-AfternoonStalkerEngine::makePlayerShoot()
+AfternoonStalkerEngine::makePlayerShoot(int shotDir)
+/*
+    Start a bullet at the player's position, in the direction specified
+    by 'shotDir', which must be RIGHT, UP, LEFT or DOWN.
+*/
 {
     static const char *infiniteBullets = getenv("INFINITEBULLETS");
 
@@ -891,10 +957,6 @@ AfternoonStalkerEngine::makePlayerShoot()
 
     playSoundEffect(playerBulletSound);
 
-    int shotDir = lastRequestedDir;
-    if (shotDir == -1)
-	shotDir = UP;
-
     Couple displacement = getCoupleFromDirection(shotDir,
 			playerSprite->getSize().x / 2 - BULLET_SPEED_FACTOR);
 
@@ -914,6 +976,14 @@ AfternoonStalkerEngine::makePlayerShoot()
 }
 
 
+void
+AfternoonStalkerEngine::layPlayerBomb()
+{
+    theBomb.init(playerSprite->getCenterPos(), 3 * 20, 32 * 4);
+    playerHasBomb = false;
+}
+
+
 Couple
 AfternoonStalkerEngine::attemptMove(const Sprite &s,
 				    const bool attemptedDirections[4],
@@ -1260,6 +1330,14 @@ AfternoonStalkerEngine::animateAutomaticCharacters()
     moveExplosionSprites(spiderExplosionSprites);
     moveExplosionSprites(batExplosionSprites);
 
+
+    // Animate the bomb explosion, if applicable:
+    if ((tickCount & 0x01) == 0)
+	for (SpriteList::iterator it = bombExplosionSprites.begin();
+				it != bombExplosionSprites.end(); it++)
+	    (**it).currentPixmapIndex ^= 1;
+
+
     bool playerDying = !playerExplosionSprites.empty();
     moveExplosionSprites(playerExplosionSprites);
 
@@ -1293,7 +1371,9 @@ AfternoonStalkerEngine::animateAutomaticCharacters()
 
 	const PixmapArray *pa = NULL;
 
-	if (numPlayerLives == 0)
+	if (sendOnlyBlueRobots)
+	    pa = &blueRobotPA;
+	else if (numPlayerLives == 0)
 	    pa = &grayRobotPA;  // only gray robots in demo mode
 	else
 	{
@@ -1417,6 +1497,26 @@ AfternoonStalkerEngine::animateAutomaticCharacters()
     }
 
 
+    /*  Show a bomb once in a while
+    */
+    if (!playerHasBomb
+		&& !theBomb.isTicking()
+		&& bombExplosionTicks == 0
+		&& timeUntilNextBomb > 0
+		&& --timeUntilNextBomb == 0)
+    {
+	Couple bombSize = bombPA.getImageSize();
+
+	Couple pos = chooseRandomAllowedTile();
+	pos += (Couple(1, 1) * TILE_SIDE - bombSize) / 2;
+	Sprite *bomb = new Sprite(bombPA,
+				pos, Couple(), Couple(),
+				Couple(2, 2), bombSize - Couple(4, 4));
+	bombSprites.push_back(bomb);
+    }
+
+
+
     /*  Stop blinking for robots when appropriate:
     */
     for (SpriteList::const_iterator itr = robotSprites.begin();
@@ -1442,6 +1542,116 @@ AfternoonStalkerEngine::animateAutomaticCharacters()
     /*  Make enemies attack the player:
     */
     makeRobotsShoot();
+
+
+    /*  Process the bomb, if applicable:
+    */
+    if (theBomb.tickDown())
+    {
+	playSoundEffect(bombExplodesSound);
+
+	bombExplosionTicks = 20;  // ticks during which explosion is shown
+
+	// Create the bomb explosion sprites:
+	Couple p = theBomb.getPos();
+	int r = theBomb.getExplosionRadius();
+	Couple size = bombExplosionPA.getImageSize();
+
+	for (int level = 0; level <= 2; level++)
+	{
+	    double angleIncr = PI / (2.5 * (1 << level));
+	    double radius = (level + 1) / 3.0 * (r - size.x / 3);
+	    for (double angle = 0.0; angle < TWO_PI; angle += angleIncr)
+	    {
+		double x = p.x + radius * cos(angle);
+		double y = p.y - radius * sin(angle);
+		Couple q = Couple(int(x), int(y));
+		q -= size / 2;
+		Sprite *s = new Sprite(bombExplosionPA, q,
+					Couple(), Couple(), Couple(), Couple());
+		bombExplosionSprites.push_back(s);
+	    }
+	}
+	Sprite *s = new Sprite(bombExplosionPA, p - size / 2,
+				Couple(), Couple(), Couple(), Couple());
+	bombExplosionSprites.push_back(s);
+    }
+
+
+    /*  Process the bomb explosion, if applicable:
+    */
+    if (bombExplosionTicks > 0)
+    {
+	if (--bombExplosionTicks == 0)
+	{
+	    deleteSprites(bombExplosionSprites);
+	    setTimeUntilNextBomb();
+	}
+	else
+	{
+	    // Kill sprites that are inside the explosion radius:
+	    Couple p = theBomb.getPos();
+	    int r = theBomb.getExplosionRadius();
+
+	    killSpritesNearPosition(robotSprites, p, r);
+	    killSpritesNearPosition(robotBulletSprites, p, r);
+	    killSpritesNearPosition(playerBulletSprites, p, r);
+	    killSpritesNearPosition(spiderSprites, p, r);
+	    killSpritesNearPosition(batSprites, p, r);
+
+	    // Kill player if inside the explosion radius:
+	    Couple v = p - playerSprite->getCenterPos();
+	    double dist = hypot(v.x, v.y);
+	    if (dist <= r)
+		makePlayerDie();
+	}
+    }
+}
+
+
+void
+AfternoonStalkerEngine::killSpritesNearPosition(
+				SpriteList &slist, Couple pos, int minDistance)
+{
+    for (SpriteList::iterator it = slist.begin(); it != slist.end(); it++)
+    {
+	Sprite &s = **it;
+	Couple v = pos - s.getCenterPos();
+	double dist = hypot(v.x, v.y);
+	if (dist <= minDistance)
+	{
+	    killEnemy(s);
+	    delete *it;
+	    *it = NULL;
+	}
+    }
+    removeNullElementsFromSpriteList(slist);
+}
+
+
+void
+AfternoonStalkerEngine::killEnemy(const Sprite &s)
+{
+    if (s.getPixmapArray() == &spiderPA)
+    {
+	playSoundEffect(spiderKilledSound);
+	createAnimalExplosion(s.getCenterPos(),
+				spiderExplosionPA, spiderExplosionSprites);
+	createScoreSprites(SPIDER_SCORE, s.getCenterPos());
+    }
+    else if (s.getPixmapArray() == &batPA)
+    {
+	playSoundEffect(batKilledSound);
+	createAnimalExplosion(s.getCenterPos(),
+				batExplosionPA, batExplosionSprites);
+	createScoreSprites(BAT_SCORE, s.getCenterPos());
+    }
+    else if (dynamic_cast<const RobotSprite *>(&s) != NULL)
+    {
+	playSoundEffect(robotHitSound);
+	createRobotExplosion(dynamic_cast<const RobotSprite &>(s));
+	createScoreSprites(getRobotScore(s), s.getCenterPos());
+    }
 }
 
 
@@ -1456,12 +1666,7 @@ AfternoonStalkerEngine::moveExplosionSprites(SpriteList &slist)
 	expl->addAccelToSpeed();
 	expl->addSpeedToPos();
 
-	if (isOutOfSetting(*expl))
-	{
-	    *it = NULL;
-	    delete expl;
-	}
-	else if (expl->getTimeToLive() > 0 && expl->decTimeToLive() == 0)
+	if (expl->getTimeToLive() > 0 && expl->decTimeToLive() == 0)
 	{
 	    *it = NULL;
 	    delete expl;
@@ -1512,7 +1717,7 @@ AfternoonStalkerEngine::moveEnemyList(SpriteList &slist, int speedFactor)
 {
     const Couple plpos = playerSprite->getPos();
 
-    bool changePixmap = (tickCount % (FRAMES_PER_SECOND / 5) == 0);
+    bool changePixmap = (tickCount % (FPS / 5) == 0);
 
     size_t batPixmapIndex;
     { 
@@ -1823,7 +2028,7 @@ AfternoonStalkerEngine::detectCollisions()
     SpriteList::iterator it;
 
 
-    /*  Human vs gun:
+    /*  Human vs gun and bomb:
     */
     if (numPlayerLives > 0)
     {
@@ -1836,10 +2041,23 @@ AfternoonStalkerEngine::detectCollisions()
 		numPlayerBullets += BULLETS_PER_GUN;
 		delete *it;
 		*it = NULL;
-		continue;
 	    }
 	}
 	removeNullElementsFromSpriteList(gunSprites);
+
+
+	for (it = bombSprites.begin(); it != bombSprites.end(); it++)
+	{
+	    if (playerSprite->collidesWithSprite(**it))
+	    {
+		playSoundEffect(gunPickupSound);  // sic
+
+		playerHasBomb = true;
+		delete *it;
+		*it = NULL;
+	    }
+	}
+	removeNullElementsFromSpriteList(bombSprites);
     }
 
 
@@ -1882,31 +2100,19 @@ AfternoonStalkerEngine::detectCollisions()
 		    {
 			createRobotExplosion(*robot);
 
-			long score = -1;
-			if (robot->getPixmapArray() == &grayRobotPA)
-			    score = 400;
-			else if (robot->getPixmapArray() == &blueRobotPA)
-			    score = 500;
-			else if (robot->getPixmapArray() == &whiteRobotPA)
-			    score = 1000;
-			else if (robot->getPixmapArray() == &blackRobotPA)
-			    score = 2000;
-			else if (robot->getPixmapArray() == &invisibleRobotPA)
-			    score = 4000;
-			else
-			    assert(false);
+			long score = getRobotScore(*robot);
 			createScoreSprites(score, robot->getCenterPos());
 
 			delete robot;
 			*itr = NULL;
 			setTimeUntilNextRobot();
-			timeUntilNextRobot += 2 * FRAMES_PER_SECOND;
+			timeUntilNextRobot += 2 * FPS;
 			    // add extra time to allow explosion to disappear
 		    }
 		    else
 		    {
 			robot->startBlinking(blinkPA,
-				    tickCount + FRAMES_PER_SECOND * 3 / 2);
+				    tickCount + FPS * 3 / 2);
 			    // initializes robot->endOfBlinkTime
 		    }
 		    
@@ -1935,11 +2141,11 @@ AfternoonStalkerEngine::detectCollisions()
 
 		createAnimalExplosion(spider->getCenterPos(),
 				spiderExplosionPA, spiderExplosionSprites);
-		createScoreSprites(100, spider->getCenterPos());
+		createScoreSprites(SPIDER_SCORE, spider->getCenterPos());
 		delete spider;
 		*its = NULL;
 		setTimeUntilNextAnimal();
-		timeUntilNextAnimal += 2 * FRAMES_PER_SECOND;
+		timeUntilNextAnimal += 2 * FPS;
 		    // add extra time to allow explosion to disappear
 
 		/*  No 'break' statement here, since the bullet is allowed
@@ -1966,11 +2172,11 @@ AfternoonStalkerEngine::detectCollisions()
 
 		createAnimalExplosion(bat->getCenterPos(),
 				    batExplosionPA, batExplosionSprites);
-		createScoreSprites(300, bat->getCenterPos());
+		createScoreSprites(BAT_SCORE, bat->getCenterPos());
 		delete bat;
 		*its = NULL;
 		setTimeUntilNextAnimal();
-		timeUntilNextAnimal += 2 * FRAMES_PER_SECOND;
+		timeUntilNextAnimal += 2 * FPS;
 		    // add extra time to allow explosion to disappear
 
 		/*  No 'break' statement here, since the bullet is allowed
@@ -2087,6 +2293,25 @@ AfternoonStalkerEngine::detectCollisions()
 }
 
 
+int
+AfternoonStalkerEngine::getRobotScore(const Sprite &robot) const
+{
+    int score = 0;
+    if (robot.getPixmapArray() == &grayRobotPA)
+	score = 400;
+    else if (robot.getPixmapArray() == &blueRobotPA)
+	score = 500;
+    else if (robot.getPixmapArray() == &whiteRobotPA)
+	score = 1000;
+    else if (robot.getPixmapArray() == &blackRobotPA)
+	score = 2000;
+    else if (robot.getPixmapArray() == &invisibleRobotPA)
+	score = 4000;
+    assert(score != 0);
+    return score;
+}
+
+
 void
 AfternoonStalkerEngine::makePlayerDie()
 {
@@ -2094,6 +2319,7 @@ AfternoonStalkerEngine::makePlayerDie()
 
     createPlayerExplosion(*playerSprite);
 
+    theBomb.reset();
 
     playerSprite->setPos(initPlayerPos);
     playerSprite->currentPixmapIndex = 0;
@@ -2112,7 +2338,7 @@ AfternoonStalkerEngine::makePlayerDie()
 void
 AfternoonStalkerEngine::paralyzePlayer()
 {
-    playerParalysisTime = FRAMES_PER_SECOND * 3;
+    playerParalysisTime = FPS * 3;
 }
 
 
@@ -2155,7 +2381,7 @@ AfternoonStalkerEngine::createAnimalExplosion(Couple pos,
 
 	Sprite *expl = new Sprite(pa, Couple(x, y), Couple(dx, dy),
 				    Couple(), Couple(), Couple());
-	expl->setTimeToLive(FRAMES_PER_SECOND * 3 / 2);
+	expl->setTimeToLive(FPS * 3 / 2);
 	slist.push_back(expl);
     }
 }
@@ -2180,7 +2406,7 @@ AfternoonStalkerEngine::createPlayerExplosion(const Sprite &s)
 	Sprite *expl = new Sprite(playerExplosionPA,
 				    Couple(x, y), Couple(dx, dy),
 				    Couple(), Couple(), Couple());
-	expl->setTimeToLive(FRAMES_PER_SECOND * 2);
+	expl->setTimeToLive(FPS * 2);
 	playerExplosionSprites.push_back(expl);
     }
 }
@@ -2269,6 +2495,7 @@ AfternoonStalkerEngine::loadPixmaps() throw(PixmapLoadError)
     loadPixmap(big_robot_bullet_1_xpm, bigRobotBulletPA, 1);
     loadPixmap(big_robot_bullet_2_xpm, bigRobotBulletPA, 2);
     loadPixmap(gun_xpm, gunPA, 0);
+    loadPixmap(bomb_xpm, bombPA, 0);
     loadPixmap(human_explosion_0_xpm, playerExplosionPA, 0);
     loadPixmap(gray_robot_0_xpm, grayRobotPA, 0);
     loadPixmap(gray_robot_1_xpm, grayRobotPA, 1);
@@ -2316,6 +2543,10 @@ AfternoonStalkerEngine::loadPixmaps() throw(PixmapLoadError)
     loadPixmap(bat_explosion_0_xpm, batExplosionPA, 0);
 
 
+    loadPixmap(bomb_explosion_0_xpm, bombExplosionPA, 0);
+    loadPixmap(bomb_explosion_1_xpm, bombExplosionPA, 1);
+
+
     /*  Score digits:
     */
     loadPixmap(digit0_xpm, digitPA, 0);
@@ -2351,7 +2582,7 @@ AfternoonStalkerEngine::restoreBackground()
 	    TileNo tileNo = tileMatrix[y][x];
 	    if (tileNo == COBWEB_TILE)  // this is a foreground tile
 		tileNo = FLOOR_TILE;  // so we don't draw it now
-	    Pixmap p = tilePA.getImage(size_t(tileNo));
+	    SDL_Surface *p = tilePA.getImage(size_t(tileNo));
 	    copyPixmap(p, dest);
 	}
     }
@@ -2364,7 +2595,7 @@ AfternoonStalkerEngine::restoreForeground()
     const size_t levelHeight = tileMatrix.size();
     const size_t levelWidth = tileMatrix[0].size();
 
-    Pixmap p = tilePA.getImage(COBWEB_TILE);  // only one foreground tile
+    SDL_Surface *p = tilePA.getImage(COBWEB_TILE);  // only one foreground tile
 
     size_t x, y;
     for (y = 0; y < levelHeight; y++)
@@ -2380,7 +2611,8 @@ AfternoonStalkerEngine::restoreForeground()
 }
 
 
-inline void
+inline
+void
 AfternoonStalkerEngine::putSprite(const Sprite &s)
 {
     copySpritePixmap(s, s.currentPixmapIndex, s.getPos());
@@ -2437,6 +2669,7 @@ AfternoonStalkerEngine::drawSprites()
     putSpriteList(spiderExplosionSprites);
     putSpriteList(batExplosionSprites);
     putSpriteList(playerExplosionSprites);
+    putSpriteList(bombExplosionSprites);
 
 
     putSpriteList(robotBulletSprites);
@@ -2447,10 +2680,20 @@ AfternoonStalkerEngine::drawSprites()
 
 
     // Make the gun(s) blink if applicable:
-    if (tickCount % FRAMES_PER_SECOND < FRAMES_PER_SECOND * 3 / 4)
+    if (tickCount % FPS < FPS * 3 / 4)
 	putSpriteList(gunSprites);
 
 
+    // Make the bomb(s) blink if applicable:
+    if (tickCount % FPS < FPS * 3 / 4)
+	putSpriteList(bombSprites);
+
+
+    // Indicate if player has a bomb:
+    if (playerHasBomb)
+	displayMessage(30, "You have a bomb. Press B to lay it, then move away!");
+
+
     // Score sprites:
     for (it = scoreSprites.begin(); it != scoreSprites.end(); it++)
 	putSprite(**it);
@@ -2474,61 +2717,33 @@ AfternoonStalkerEngine::drawScoreBoard()
 
 
 void
-AfternoonStalkerEngine::displayPauseMessage(bool display)
-/*  Displays the pause message if 'display' is true, or erases the
-    corresponding region if 'display' is false.
-*/
+AfternoonStalkerEngine::displayPauseMessage()
 {
-    const char *s = (display ? "PAUSED -- press P to resume" : "");
-    displayMessage(s);
+    displayMessage(0, "PAUSED -- press P to resume");
 }
 
 
 void
-AfternoonStalkerEngine::displayStartMessage(bool display)
-/*  Displays the start message if 'display' is true, or erases the
-    corresponding region if 'display' is false.
-*/
+AfternoonStalkerEngine::displayStartMessage()
 {
-    if (!display)
-	displayMessage("");
-    else
-    {
-	static unsigned long firstTick = tickCount;
+    displayMessage(0, "Afternoon Stalker " VERSION " - by Pierre Sarrazin");
+    displayMessage(1, "Move with arrow keys - shoot with Ctrl key");
+    displayMessage(2, "Also shoot with H, J, K and L keys");
+    displayMessage(3, "Press SPACE to start - press Escape to quit");
 
-	const char *msg = "";
-	switch ((tickCount - firstTick) / 32 % 3)
-	{
-	    case 0:
-		msg = "Afternoon Stalker " VERSION " - by Pierre Sarrazin";
-		break;
-	    case 1:
-		msg = "Move with arrow keys - shoot with Ctrl key";
-		break;
-	    case 2:
-		msg = "Press SPACE to start";
-		break;
-	    default:
-		assert(false);
-		msg = "";
-	}
-
-	displayMessage(msg);
-    }
+    if (useSound && theSoundMixer == NULL)
+	displayMessage(4, "*** Sound device not available ***");
 }
 
 
 void
-AfternoonStalkerEngine::displayMessage(const char *msg)
-/*  Displays the start message if 'display' is true, or erases the
-    corresponding region if 'display' is false.
-*/
+AfternoonStalkerEngine::displayMessage(int row, const char *msg)
 {
     static const Couple fontdim = getFontDimensions();
     
     const size_t len = strlen(msg);
     int x = (theScreenSizeInPixels.x - len * fontdim.x) / 2;
-    int y = 6;
+    int y = 6 + row * fontdim.y;
 
     writeString(msg, Couple(x, y));
 }
@@ -2544,7 +2759,8 @@ AfternoonStalkerEngine::addToNumPlayerLives(int n)
 void
 AfternoonStalkerEngine::playSoundEffect(SoundMixer::Chunk &wb)
 {
-    /*  'theSoundMixer' is still NULL if the sound effects were not wanted.
+    /*  'theSoundMixer' is still NULL if the sound effects were not wanted
+	or the sound device is not available.
     */
     if (theSoundMixer != NULL)
     {
