@@ -1,8 +1,8 @@
-/*  $Id: AfternoonStalkerEngine.h,v 1.15 2003/01/04 11:18:16 sarrazip Exp $
+/*  $Id: AfternoonStalkerEngine.h,v 1.23 2004/05/02 04:55:46 sarrazip Exp $
     AfternoonStalkerEngine.h - A robot-killing video game engine.
 
     afternoonstalker - A robot-killing video game.
-    Copyright (C) 2001, 2002, 2003 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -25,16 +25,17 @@
 
 #define PAUL 12
 
-#include <gengameng/GameEngine.h>
-#include <gengameng/Sprite.h>
-
-#include "SoundMixer.h"
+#include <flatzebra/GameEngine.h>
+#include <flatzebra/Sprite.h>
+#include <flatzebra/SoundMixer.h>
+#include <flatzebra/KeyState.h>
 
 class RobotSprite;
 
 #include <string>
+#include <vector>
 
-using namespace std;
+using namespace flatzebra;
 
 
 class AfternoonStalkerEngine : public GameEngine
@@ -43,21 +44,30 @@ class AfternoonStalkerEngine : public GameEngine
 {
 public:
 
-    typedef PixmapArray::Pixmap Pixmap;
-
-
     enum { RIGHT, UP, LEFT, DOWN };
     /*	Possible directions of movement, in trigonometric order.
 	This enumeration is garanteed to start at zero.
 	The order of the constants is guaranteed, so they are 0, 1, 2, 3.
     */
 
-    AfternoonStalkerEngine(const string &windowManagerCaption,
-			    bool useSound, bool pedantic, bool fullScreen)
-						throw(int, string);
+    AfternoonStalkerEngine(const std::string &windowManagerCaption,
+			    bool useSound,
+			    bool pedantic,
+			    bool fullScreen,
+			    bool onlyBlueRobots,
+			    const std::string &setFilename)
+				throw(int, std::string);
     /*  See base class.
 
 	'windowManagerCaption' must contain the title to give to the window.
+	'useSound' determines if sound effects are produced.
+	'pedantic' determines if a special sound effect is played when
+	a bullet shot by the player does not hit anything.
+	'fullScreen' attempts to display the fame in full screen mode
+	instead of using an ordinary window.
+	'onlyBlueRobots' sets a practice mode where only blue robots
+	are sent out (they are smarter than gray robots, but only
+	require one bullet to be killed).
 
 	Throws a string or an integer code if an error occurs.
     */
@@ -115,8 +125,8 @@ private:
 	BUNKER_DOOR_TILE
     };
 
-    typedef vector<TileNo> TileMatrixRow;
-    typedef vector<TileMatrixRow> TileMatrix;
+    typedef std::vector<TileNo> TileMatrixRow;
+    typedef std::vector<TileMatrixRow> TileMatrix;
 
 
     enum Privilege
@@ -129,6 +139,52 @@ private:
     };
 
 
+    class Bomb
+    {
+    public:
+	Couple pos;
+	int ticksBeforeExplosion;
+	int explosionRadius;
+
+	Bomb()
+	  : pos(), ticksBeforeExplosion(0), explosionRadius(0) {}
+
+	void init(Couple pos, int ticksBeforeExplosion, int explosionRadius)
+	{
+	    this->pos = pos;
+	    this->ticksBeforeExplosion = ticksBeforeExplosion;
+	    this->explosionRadius = explosionRadius;
+	}
+
+	bool tickDown()
+	{
+	    if (ticksBeforeExplosion == 0)
+		return false;
+	    ticksBeforeExplosion--;
+	    return (ticksBeforeExplosion == 0);
+	}
+
+	void reset()
+	{
+	    ticksBeforeExplosion = 0;
+	}
+
+	bool isTicking() const
+	{
+	    return (ticksBeforeExplosion > 0);
+	}
+
+	Couple getPos() const
+	{
+	    return pos;
+	}
+
+	int getExplosionRadius() const
+	{
+	    return explosionRadius;
+	}
+    };
+
 
     ///////////////////////////////////////////////////////////////////////////
     //
@@ -139,6 +195,7 @@ private:
 
     bool paused;
     unsigned long tickCount;
+    bool useSound;
 
 
     // SDL color values (see SDL_MapRGB()):
@@ -169,6 +226,7 @@ private:
     int lastPlayerDir;
     int lastRequestedDir;
     long playerParalysisTime;
+    bool playerHasBomb;
 
     PixmapArray playerBulletPA;
     SpriteList playerBulletSprites;
@@ -180,6 +238,15 @@ private:
     PixmapArray gunPA;
     SpriteList gunSprites;
 
+    PixmapArray bombPA;
+    SpriteList bombSprites;
+    long timeUntilNextBomb;
+
+    Bomb theBomb;
+    int bombExplosionTicks;
+    PixmapArray bombExplosionPA;
+    SpriteList bombExplosionSprites;
+
 
     /*  ENEMIES:
 
@@ -193,6 +260,7 @@ private:
     PixmapArray blinkPA;
     SpriteList robotSprites;
     long timeUntilNextRobot;
+    bool sendOnlyBlueRobots;
     Couple initRobotPos;
     size_t numCreatedRobots;
     size_t maxNumRobots;
@@ -228,14 +296,19 @@ private:
 
     /*  KEYBOARD COMMANDS:
     */
-    bool escapePressed;
-    bool leftArrowPressed;
-    bool rightArrowPressed;
-    bool upArrowPressed;
-    bool downArrowPressed;
-    bool spacePressed, spacePressedBefore;
-    bool ctrlPressed, ctrlPressedBefore;
-    bool letterPPressed, letterPPressedBefore;
+    KeyState quitKS;
+    KeyState leftKS;
+    KeyState rightKS;
+    KeyState upKS;
+    KeyState downKS;
+    KeyState startKS;
+    KeyState shootKS;
+    KeyState pauseKS;
+    KeyState bombKS;
+    KeyState shootLeftKS;
+    KeyState shootRightKS;
+    KeyState shootUpKS;
+    KeyState shootDownKS;
 
 
     /*  SOUND EFFECTS:
@@ -250,6 +323,7 @@ private:
     SoundMixer::Chunk batKilledSound;
     SoundMixer::Chunk spiderKilledSound;
     SoundMixer::Chunk newLifeSound;
+    SoundMixer::Chunk bombExplodesSound;
     SoundMixer::Chunk missedSound;
 
 
@@ -259,7 +333,9 @@ private:
     //
     //
     bool animatePlayer();
-    void makePlayerShoot();
+    int getRobotScore(const Sprite &robot) const;
+    void makePlayerShoot(int shotDir);
+    void layPlayerBomb();
     Couple attemptMove(const Sprite &s,
 			const bool attemptedDirections[4],
 			int speedFactor) const;
@@ -275,13 +351,15 @@ private:
     Couple getDistanceToPerfectPos(const Sprite &s) const;
     bool positionIsAllowed(int direction, Privilege priv,
 				Couple pos, Couple size) const;
-    pair<TileNo, TileNo> getSideTilesAtPos(
+    std::pair<TileNo, TileNo> getSideTilesAtPos(
 				int direction, Couple pos, Couple size) const;
     TileNo getTileAtPos(Couple pos) const;
     void setTileAtPos(Couple pos, TileNo t);
 
     void animateTemporarySprites(SpriteList &slist) const;
     void animateAutomaticCharacters();
+    void killSpritesNearPosition(
+			SpriteList &slist, Couple pos, int minDistance);
     void moveExplosionSprites(SpriteList &slist);
     bool isOutOfSetting(const Sprite &s) const;
     size_t getNumToughRobots() const;
@@ -300,6 +378,7 @@ private:
     void createRobotExplosion(const RobotSprite &r);
     void createAnimalExplosion(Couple pos,
     				const PixmapArray &pa, SpriteList &slist);
+    void killEnemy(const Sprite &s);
     void createPlayerExplosion(const Sprite &s);
     void addToScore(long n);
     void createScoreSprites(long n, Couple center);
@@ -310,18 +389,24 @@ private:
     void putSpriteList(const SpriteList &slist);
     void drawSprites();
     void drawScoreBoard();
-    void displayPauseMessage(bool display);
-    void displayStartMessage(bool display);
-    void displayMessage(const char *msg);
+    void displayPauseMessage();
+    void displayStartMessage();
+    void displayMessage(int row, const char *msg);
     void addToNumPlayerLives(int n);
 
-    void displayErrorMessage(const string &msg) throw();
+    void displayErrorMessage(const std::string &msg) throw();
     void initializeLevel();
     void initializePlayerLife();
+    void setTimeUntilNextBomb();
     void setTimeUntilNextRobot();
     void setTimeUntilNextAnimal();
-    void doOneTimeInitializations(bool useSound, bool pedantic) throw(string);
-    void loadLevel(int levelNo, const string &setFilename) throw(string);
+    void doOneTimeInitializations(
+			bool useSound,
+			bool pedantic,
+			const std::string &setFilename)
+				throw(std::string);
+    void loadLevel(int levelNo,
+    			const std::string &setFilename) throw(std::string);
 
     void playSoundEffect(SoundMixer::Chunk &wb);
 
