@@ -1,8 +1,8 @@
-/*  $Id: RobotSprite.h,v 1.4 2003/01/04 11:18:16 sarrazip Exp $
+/*  $Id: RobotSprite.h,v 1.7 2004/05/02 04:55:46 sarrazip Exp $
     RobotSprite.h - Sprite that represents a robot.
 
     afternoonstalker - A robot-killing video game.
-    Copyright (C) 2001, 2002, 2003 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -23,7 +23,9 @@
 #ifndef _H_RobotSprite
 #define _H_RobotSprite
 
-#include <gengameng/Sprite.h>
+#include <flatzebra/Sprite.h>
+
+using namespace flatzebra;
 
 
 class RobotSprite : public Sprite
