@@ -1,8 +1,8 @@
-/*  $Id: sdlmain.cpp,v 1.7 2003/01/04 11:18:16 sarrazip Exp $
+/*  $Id: sdlmain.cpp,v 1.11 2004/05/02 04:55:46 sarrazip Exp $
     sdlmain.cpp - main() function for Afternoon Stalker
 
     afternoonstalker - A robot-killing video game.
-    Copyright (C) 2001, 2002, 2003 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -27,16 +27,17 @@
 #include <time.h>
 #include <stdio.h>
 
-#ifdef __GNUG__
+#ifdef HAVE_GETOPT_LONG
     #include <getopt.h>
 #endif
 
+#include <iostream>
 #include <string>
 
 using namespace std;
 
 
-#ifdef __GNUG__
+#ifdef HAVE_GETOPT_LONG
 static struct option knownOptions[] =
 {
     { "help",          no_argument,       NULL, 'h' },
@@ -45,45 +46,50 @@ static struct option knownOptions[] =
     { "no-sound",      no_argument,       NULL, 'n' },
     { "full-screen",   no_argument,       NULL, 'f' },
     { "pedantic",      no_argument,       NULL, 'p' },
+    { "only-blue",     no_argument,       NULL, 'b' },
+    { "set-file",      required_argument, NULL, 's' },
 
     { NULL, 0, NULL, 0 }  // marks the end
 };
 
 
 static void
-displayVersionNo()
+displayVersionNo(ostream &out)
 {
-    printf("%s %s\n", PACKAGE, VERSION);
+    out << PACKAGE << " " << VERSION << "\n";
 }
 
 
 static void
-displayHelp()
+displayHelp(ostream &out)
 {
-    printf("\n");
+    out << "\n";
 
-    displayVersionNo();
+    displayVersionNo(out);
 
-    printf(
+    out <<
 "\n"
 "Copyright (C) 2001, 2002, 2003 Pierre Sarrazin <http://sarrazip.com/>\n"
 "This program is free software; you may redistribute it under the terms of\n"
 "the GNU General Public License.  This program has absolutely no warranty.\n"
-    );
+    ;
 
-    printf(
+    out <<
 "\n"
 "Known options:\n"
 "--help             Display this help page and exit\n"
 "--version          Display this program's version number and exit\n"
-"--ms-per-frame=N   N milliseconds per animation frame [default is 55]\n"
+"--ms-per-frame=N   N milliseconds per animation frame [default: 55]\n"
 "                   Min. 1, max. 1000.  50 ms means 20 frames per second\n"
-"--no-sound         Disable sound effects [default is to enable them]\n"
-"--full-screen      Attempt to use full screen mode [default is window mode]\n"
+"--no-sound         Disable sound effects [enabled]\n"
+"--full-screen      Attempt to use full screen mode [window mode]\n"
+"--only-blue        Practice mode: only send blue robots\n"
+"--set-file=F       Use file F as the set description file [level1.set]\n"
+"                   F must be in " PKGDATADIR "\n"
 "\n"
-    );
+    ;
 }
-#endif  /* __GNUG__ */
+#endif  /* HAVE_GETOPT_LONG */
 
 
 int main(int argc, char *argv[])
@@ -94,8 +100,10 @@ int main(int argc, char *argv[])
     bool useSound = true;
     bool pedantic = false;
     bool fullScreen = false;
+    bool onlyBlueRobots = false;
+    string setFilename = "level1.set";
 
-    #ifdef __GNUG__
+    #ifdef HAVE_GETOPT_LONG
 
     /*  Interpret the command-line options:
     */
@@ -115,8 +123,8 @@ int main(int argc, char *argv[])
 		    long n = strtol(optarg, NULL, 10);
 		    if (errno == ERANGE || n < 1 || n > 1000)
 		    {
-			printf("Invalid number of ms per frame.\n");
-			displayHelp();
+			cerr << "Invalid number of ms per frame.\n";
+			displayHelp(cerr);
 			return EXIT_FAILURE;
 		    }
 
@@ -132,22 +140,33 @@ int main(int argc, char *argv[])
 		fullScreen = true;
 		break;
 
-	    case 'v':
-		displayVersionNo();
-		return EXIT_FAILURE;
-
 	    case 'p':
 		pedantic = true;
 		break;
 
+	    case 'b':
+		onlyBlueRobots = true;
+		break;
+
+	    case 's':
+		setFilename = optarg;
+		break;
+
+	    case 'v':
+		displayVersionNo(cout);
+		return EXIT_SUCCESS;
+
 	    case 'h':
+		displayHelp(cout);
+		return EXIT_SUCCESS;
+
 	    default:
-		displayHelp();
+		displayHelp(cerr);
 		return EXIT_FAILURE;
 	}
     } while (c != EOF && c != '?');
 
-    #endif  /* __GNUG__ */
+    #endif  /* HAVE_GETOPT_LONG */
 
 
     /*  Initialize the random number generator:
@@ -155,8 +174,8 @@ int main(int argc, char *argv[])
     const char *s = getenv("SEED");
     unsigned int seed = (s != NULL ? atol(s) : time(NULL));
     #ifndef NDEBUG
-    fprintf(stderr, "seed = %u\n", seed);
-    fprintf(stderr, "ms-per-frame = %d\n", millisecondsPerFrame);
+    cerr << "seed = " << seed << "\n";
+    cerr << "ms-per-frame = " << millisecondsPerFrame << "\n";
     #endif
     srand(seed);
 
@@ -164,17 +183,19 @@ int main(int argc, char *argv[])
     try
     {
 	AfternoonStalkerEngine theAfternoonStalkerEngine(
-			"Afternoon Stalker", useSound, pedantic, fullScreen);
+			"Afternoon Stalker",
+			useSound, pedantic, fullScreen, onlyBlueRobots,
+			setFilename);
 	theAfternoonStalkerEngine.run(millisecondsPerFrame);
     }
     catch (const string &s)
     {
-	fprintf(stderr, "%s: engine init error: %s\n", PACKAGE, s.c_str());
+	cerr << PACKAGE << ": engine init error: " << s << "\n";
 	return EXIT_FAILURE;
     }
     catch (int e)
     {
-	fprintf(stderr, "%s: init error # %d\n", PACKAGE, e);
+	cerr << PACKAGE << ": init error # " << e << "\n";
 	return EXIT_FAILURE;
     }
 
